@@ -60,10 +60,12 @@ MetaQC <- function(DList, GList, isParallel=FALSE, nCores=NULL, useCache=TRUE, f
 				}
 				
 				.Initialize <- function(., .DList, .GList, .filterGenes, .resp.type) {
-					stopifnot(all(length(names(.DList))>0) & all(!duplicated(names(.DList))))
-					#stopifnot(all(sapply(.DList, function(x) length(table(colnames(x))))==2)) #currently support only two classes
-					
-					#.$.DListF0 <- 
+#					if(length(.DList)<4)
+#						stop("DList should have more than 3 data sets")
+					if(any(length(names(.DList))==0))
+						stop("DList should have names for all data sets")
+					if(any(duplicated(names(.DList))))
+						stop("DList should have unique names for all data sets")
 					
 					.$.resp.type <- .resp.type
 					
@@ -160,7 +162,9 @@ MetaQC <- function(DList, GList, isParallel=FALSE, nCores=NULL, useCache=TRUE, f
 								mean(as.dist(abs(cor(t(.d[.g,]), use="pairwise.complete.obs", method=.$.method.cor)^.$.l.norm)))^(1/.$.l.norm)
 						}
 						
-						.pathList <- foreach(j=iter(.pathMat[,!duplicated(colSums(.pathMat))],by="col")) %dopar% {
+						.pathMat.dupCols <- duplicated(colSums(.pathMat))
+						
+						.pathList <- foreach(j=iter(.pathMat[,!.pathMat.dupCols],by="col")) %dopar% {
 							which(j==1)
 						}
 						names(.pathList) <- sapply(.pathList,length)
@@ -182,9 +186,11 @@ MetaQC <- function(DList, GList, isParallel=FALSE, nCores=NULL, useCache=TRUE, f
 							return(.res)
 						}
 						
-						.dupNum <- colSums(.pathMat)[duplicated(colSums(.pathMat))]
-						.ScoresNullDist <- cbind(.ScoresNullDist, sapply(.dupNum, function(dn) .ScoresNullDist[,as.character(dn)]))
-						.ScoresNullDist <- rbind(.Scores, .ScoresNullDist)
+						if(sum(.pathMat.dupCols) > 0) {
+							.dupNum <- colSums(.pathMat)[duplicated(colSums(.pathMat))]
+							.ScoresNullDist <- cbind(.ScoresNullDist, sapply(.dupNum, function(dn) .ScoresNullDist[,as.character(dn)]))
+						} else
+							.ScoresNullDist <- rbind(.Scores, .ScoresNullDist)
 						
 						.ScoresNullDist <- foreach(w=iter(.ScoresNullDist, by="col"), .combine=cbind) %dopar% {
 							(.B+2 - rank(w)) / (.B+1)
@@ -411,7 +417,7 @@ MetaQC <- function(DList, GList, isParallel=FALSE, nCores=NULL, useCache=TRUE, f
 					names(.IScores0) <- labels(.dist)
 					printLog("IQC Finished", .$.verbose)
 					
-					.$.IScores <- 1-pnorm( qnorm(.IScores0, qnorm(0.95), 1), -qnorm(0.95), 1)
+					.$.IScores <- pmax(1-pnorm( qnorm(.IScores0, qnorm(0.95), 1), -qnorm(0.95), 1), 1e-20)
 				}
 				
 				.CalcDistOfStudies <- function(.) {
